@@ -28,6 +28,11 @@
 #' To avoid overfitting in feature allocation estimation, it is recommended that
 #' \code{"maxSize"} be close the mean number of features (i.e., columns) in the
 #' feature allocations that generated the \code{expectedPairwiseAllocationMatrix}.
+#' @param maxScans The maximum number of reallocation scans after the intial allocation.
+#' The actual number of scans may be less than \code{maxScans} since the algorithm stops
+#' if the result does not change between scans.
+#' @param multicore Logical indicating whether computations should take advantage of
+#' multiple CPU cores.
 #'
 #' @return A clustering (as a vector of cluster labels) or a feature allocation (as a binary
 #' matrix of feature indicators).
@@ -53,7 +58,7 @@
 
 salso <- function(expectedPairwiseAllocationMatrix, structure=c("clustering","featureAllocation")[1],
                   loss=c("squaredError","absoluteError","binder","lowerBoundVariationOfInformation")[1],
-                  nCandidates=100, budgetInSeconds=10, maxSize=0) {
+                  nCandidates=100, budgetInSeconds=10, maxSize=0, maxScans=10, multicore=TRUE) {
   if ( identical(structure,"clustering") ) doClustering <- TRUE
   else if ( identical(structure,"featureAllocation") ) doClustering <- FALSE
   else stop("'structure' must be either 'clustering' or 'featureAllocation'.")
@@ -67,18 +72,21 @@ salso <- function(expectedPairwiseAllocationMatrix, structure=c("clustering","fe
       stop("'loss' should be 'squaredError' or 'absoluteError' when 'structure' is 'featureAllocation'.")
   }
   nCandidates <- as.integer(nCandidates[1])
-  budgetInSeconds <- as.integer(budgetInSeconds[1])
+  budgetInSeconds <- as.double(budgetInSeconds[1])
   maxSize <- as.integer(maxSize[1])
+  maxScans <- as.integer(maxScans[1])
+  multicore <- as.logical(multicore[1])
   result <- if ( doClustering ) {
-    ref <- s$.ClusteringSummary$sequentiallyAllocatedLatentStructureOptimization(nCandidates,budgetInSeconds,epam,maxSize,loss)
+    ref <- s$.ClusteringSummary$sequentiallyAllocatedLatentStructureOptimization(nCandidates,budgetInSeconds,epam,maxSize,maxScans,multicore,loss)
     ref$"_1"()$toLabels()+1L
   } else {
-    ref <- s$.FeatureAllocationSummary$sequentiallyAllocatedLatentStructureOptimization(nCandidates,budgetInSeconds,epam,maxSize,loss)
+    ref <- s$.FeatureAllocationSummary$sequentiallyAllocatedLatentStructureOptimization(nCandidates,budgetInSeconds,epam,maxSize,multicore,loss)
     result <- scalaConvert.featureAllocation(ref$"_1"(),withParameters=FALSE)
     attr(result,"scalaReference") <- NULL
     result
   }
-  attr(result,"nCandidates") <- ref$"_2"()
+  attr(result,"nScans") <- ref$"_2"()
+  attr(result,"nCandidates") <- ref$"_3"()
   result
 }
 
